// Websocket based client for koheron-server...
// (c) Koheron
"use strict";
var Driver = (function () {
    function Driver(name, id, cmds) {
        this.name = name;
        this.id = id;
        this.cmds = cmds;
    }
    Driver.prototype.getCmdRef = function (cmdName) {
        for (var _i = 0, _a = this.cmds; _i < _a.length; _i++) {
            var cmd = _a[_i];
            if (cmdName === cmd.name) {
                return cmd.id;
            }
        }
        throw new ReferenceError(cmdName + ': command not found');
    };
    Driver.prototype.getCmds = function () {
        var cmdsDict = {};
        for (var _i = 0, _a = this.cmds; _i < _a.length; _i++) {
            var cmd = _a[_i];
            cmdsDict[cmd.name] = cmd;
        }
        return cmdsDict;
    };
    return Driver;
}());
var WebSocketPool = (function () {
    function WebSocketPool(poolSize, url, onOpenCallback) {
        var _this = this;
        this.poolSize = poolSize;
        this.url = url;
        this.onOpenCallback = onOpenCallback;
        this.poolSize = poolSize;
        this.url = url;
        this.pool = [];
        this.freeSockets = [];
        this.dedicatedSockets = [];
        this.socketCounter = 0;
        this.exiting = false;
        for (var i = 0, end = this.poolSize - 1, asc = 0 <= end; asc ? i <= end : i >= end; asc ? i++ : i--) {
            var websocket = this._newWebSocket();
            this.pool.push(websocket);
            websocket.onopen = function (evt) {
                return _this.waitForConnection(websocket, 100, function () {
                    console.assert(websocket.readyState === 1, 'Websocket not ready');
                    _this.freeSockets.push(_this.socketCounter);
                    if (_this.socketCounter === 0) {
                        onOpenCallback();
                    }
                    websocket.ID = _this.socketCounter;
                    websocket.onclose = function (evt) {
                        setTimeout(function () { location.reload(); }, 1000);
                    };
                    websocket.onerror = function (evt) {
                        console.error("error: " + evt.data + "\n");
                        return websocket.close();
                    };
                    return _this.socketCounter++;
                });
            };
        }
    }
    WebSocketPool.prototype._newWebSocket = function () {
        var websocket;
        if (typeof window !== 'undefined' && window !== null) {
            websocket = new WebSocket(this.url);
        }
        else {
            var clientConfig = {};
            clientConfig.fragmentOutgoingMessages = false;
            var __WebSocket = require('websocket').w3cwebsocket;
            websocket = new __WebSocket(this.url, null, null, null, null, clientConfig);
        }
        websocket.binaryType = 'arraybuffer';
        return websocket;
    };
    WebSocketPool.prototype.waitForConnection = function (websocket, interval, callback) {
        var _this = this;
        if (websocket.readyState === 1) {
            // If the client has been set to exit while establishing the connection
            // we don't initialize the socket
            if (this.exiting) {
                return;
            }
            return callback();
        }
        else {
            return setTimeout(function () {
                return _this.waitForConnection(websocket, interval, callback);
            }, interval);
        }
    };
    WebSocketPool.prototype.getSocket = function (sockid) {
        if (this.exiting) {
            return null;
        }
        if (!(this.freeSockets.some(function (x) { return x == sockid; })) && (sockid >= 0) && (sockid < this.poolSize)) {
            return this.pool[sockid];
        }
        else {
            throw new ReferenceError("Socket ID " + sockid.toString() + " not available");
        }
    };
    // Obtain a socket for a dedicated task.
    // This function is provided for long term use.
    // A new socket is opened and returned to the caller.
    // This socket is not shared and can be used for a
    // dedicated task.
    // TODO: Caller close dedicated socket
    WebSocketPool.prototype.getDedicatedSocket = function (callback) {
        var _this = this;
        var websocket = this._newWebSocket();
        this.dedicatedSockets.push(websocket);
        return websocket.onopen = function (evt) {
            return _this.waitForConnection(websocket, 100, function () {
                console.assert(websocket.readyState === 1, 'Websocket not ready');
                websocket.onclose = function (evt) {
                    var idx = _this.dedicatedSockets.indexOf(websocket);
                    if (idx > -1) {
                        return _this.dedicatedSockets.splice(idx, 1);
                    }
                };
                websocket.onerror = function (evt) {
                    console.error("error: " + evt.data + "\n");
                    return websocket.close();
                };
                return callback(websocket);
            });
        };
    };
    // Retrieve a socket from the pool.
    // This function is provided for short term socket use
    // and the socket must be given back to the pool by
    // calling freeeSocket.
    WebSocketPool.prototype.requestSocket = function (callback) {
        var _this = this;
        if (this.exiting) {
            callback(-1);
            return;
        }
        if ((this.freeSockets != null) && (this.freeSockets.length > 0)) {
            var sockid_1 = this.freeSockets[this.freeSockets.length - 1];
            this.freeSockets.pop();
            return this.waitForConnection(this.pool[sockid_1], 100, function () {
                console.assert(_this.pool[sockid_1].readyState === 1, 'Websocket not ready');
                return callback(sockid_1);
            });
        }
        else {
            console.assert(this.freeSockets.length === 0, 'Non empty freeSockets');
            return setTimeout(function () {
                return _this.requestSocket(callback);
            }, 100);
        }
    };
    // Give back a socket to the pool
    WebSocketPool.prototype.freeSocket = function (sockid) {
        if (this.exiting) {
            return;
        }
        if (!(this.freeSockets.some(function (x) { return x == sockid; })) && (sockid >= 0) && (sockid < this.poolSize)) {
            return this.freeSockets.push(sockid);
        }
        else {
            if (sockid != null) {
                return console.error("Invalid Socket ID " + sockid.toString());
            }
            else {
                return console.error('Undefined Socket ID');
            }
        }
    };
    WebSocketPool.prototype.exit = function () {
        this.exiting = true;
        this.freeSockets = [];
        this.socketCounter = 0;
        for (var _i = 0, _a = this.pool; _i < _a.length; _i++) {
            var websocket = _a[_i];
            websocket.close();
        }
        return this.dedicatedSockets.map(function (socket) {
            return socket.close();
        });
    };
    return WebSocketPool;
}());
// === Helper functions to build binary buffer ===
var appendInt8 = function (buffer, value) {
    buffer.push(value & 0xff);
    return 1;
};
var appendUint8 = function (buffer, value) {
    buffer.push(value & 0xff);
    return 1;
};
var appendInt16 = function (buffer, value) {
    buffer.push((value >> 8) & 0xff);
    buffer.push(value & 0xff);
    return 2;
};
var appendUint16 = function (buffer, value) {
    buffer.push((value >> 8) & 0xff);
    buffer.push(value & 0xff);
    return 2;
};
var appendInt32 = function (buffer, value) {
    buffer.push((value >> 24) & 0xff);
    buffer.push((value >> 16) & 0xff);
    buffer.push((value >> 8) & 0xff);
    buffer.push(value & 0xff);
    return 4;
};
var appendUint32 = function (buffer, value) {
    buffer.push((value >> 24) & 0xff);
    buffer.push((value >> 16) & 0xff);
    buffer.push((value >> 8) & 0xff);
    buffer.push(value & 0xff);
    return 4;
};
var floatToBytes = function (f) {
    var buf = new ArrayBuffer(4);
    (new Float32Array(buf))[0] = f;
    return (new Uint32Array(buf))[0];
};
var bytesTofloat = function (bytes) {
    var buffer = new ArrayBuffer(4);
    (new Uint32Array(buffer))[0] = bytes;
    return new Float32Array(buffer)[0];
};
var appendFloat32 = function (buffer, value) { return appendUint32(buffer, floatToBytes(value)); };
var appendFloat64 = function (buffer, value) {
    var buf = new ArrayBuffer(8);
    (new Float64Array(buf))[0] = value;
    appendUint32(buffer, (new Uint32Array(buf, 4))[0]);
    appendUint32(buffer, (new Uint32Array(buf, 0))[0]);
    console.assert(buf.byteLength === 8, 'Invalid float64 size');
    return buf.byteLength;
};
var appendArray = function (buffer, array) {
    var bytes = new Uint8Array(array.buffer);
    return bytes.map(function (_byte) { return buffer.push(_byte); });
};
var appendVector = function (buffer, array) {
    appendUint32(buffer, array.buffer.byteLength);
    var bytes = new Uint8Array(array.buffer);
    return bytes.map(function (_byte) { return buffer.push(_byte); });
};
var appendString = function (buffer, str) {
    appendUint32(buffer, str.length);
    for (var i = 0; i < str.length; i++) {
        buffer.push(str.charCodeAt(i));
    }
};
var isStdArray = function (type) { return type.split('<')[0].trim() === 'std::array'; };
var isStdVector = function (type) { return type.split('<')[0].trim() === 'std::vector'; };
var isStdString = function (type) { return type.trim() === 'std::string'; };
var isStdTuple = function (type) { return type.split('<')[0].trim() === 'std::tuple'; };
var getStdVectorType = function (type) { return type.split('<')[1].split('>')[0].trim(); };
function Command(devId, cmd) {
    var params = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        params[_i - 2] = arguments[_i];
    }
    var buffer = [];
    appendUint32(buffer, 0); // RESERVED
    appendUint16(buffer, devId);
    appendUint16(buffer, cmd.id);
    if (cmd.args.length === 0) {
        return {
            devid: devId,
            cmd: cmd,
            data: new Uint8Array(buffer)
        };
    }
    if (cmd.args.length !== params.length) {
        throw new Error("Invalid number of arguments. Expected " + cmd.args.length + " receive " + params.length + ".");
    }
    var buildPayload = function (args, params) {
        if (args == null) {
            args = [];
        }
        var payload = [];
        for (var i = 0; i < args.length; i++) {
            var arg = args[i];
            switch (arg.type) {
                case 'uint8_t':
                    appendUint8(payload, params[i]);
                    break;
                case 'int8_t':
                    appendInt8(payload, params[i]);
                    break;
                case 'uint16_t':
                    appendUint16(payload, params[i]);
                    break;
                case 'int16_t':
                    appendInt16(payload, params[i]);
                    break;
                case 'uint32_t':
                    appendUint32(payload, params[i]);
                    break;
                case 'int32_t':
                    appendInt32(payload, params[i]);
                    break;
                case 'float':
                    appendFloat32(payload, params[i]);
                    break;
                case 'double':
                    appendFloat64(payload, params[i]);
                    break;
                case 'bool':
                    if (params[i]) {
                        appendUint8(payload, 1);
                    }
                    else {
                        appendUint8(payload, 0);
                    }
                    break;
                default:
                    if (isStdArray(arg.type)) {
                        appendArray(payload, params[i]);
                    }
                    else if (isStdVector(arg.type)) {
                        appendVector(payload, params[i]);
                    }
                    else if (isStdString(arg.type)) {
                        appendString(payload, params[i]);
                    }
                    else {
                        throw new TypeError("Unknown type " + arg.type);
                    }
            }
        }
        return payload;
    };
    return {
        devid: devId,
        cmd: cmd,
        data: new Uint8Array(buffer.concat(buildPayload(cmd.args, params)))
    };
}
var Client = (function () {
    function Client(IP, websockPoolSize) {
        this.IP = IP;
        this.websockPoolSize = websockPoolSize;
        if (websockPoolSize == null) {
            websockPoolSize = 5;
        }
        this.websockPoolSize = websockPoolSize;
        this.url = "ws://" + IP + ":8080";
        this.driversList = [];
    }
    Client.prototype.init = function (callback) {
        return this.websockpool = new WebSocketPool(this.websockPoolSize, this.url, (function () {
            return this.loadCmds(callback);
        }.bind(this)));
    };
    Client.prototype.exit = function () {
        this.websockpool.exit();
        return delete this.websockpool;
    };
    // ------------------------
    //  Send
    // ------------------------
    Client.prototype.send = function (cmd) {
        var _this = this;
        if (this.websockpool === null || typeof this.websockpool === 'undefined') {
            return;
        }
        this.websockpool.requestSocket(function (sockid) {
            if (sockid < 0) {
                return;
            }
            var websocket = _this.websockpool.getSocket(sockid);
            websocket.send(cmd.data);
            if (_this.websockpool !== null && typeof _this.websockpool !== 'undefined') {
                _this.websockpool.freeSocket(sockid);
            }
        });
    };
    // ------------------------
    //  Receive
    // ------------------------
    Client.prototype.redirectError = function (errTest, errMsg, onSuccess, onError) {
        if (errTest) {
            if (onError === null) {
                throw new TypeError(errMsg);
            }
            else {
                return onError(errMsg);
            }
        }
        else {
            return onSuccess();
        }
    };
    Client.prototype.getPayload = function (mode, evt) {
        var buffer, dvBuff, i, len;
        var dv = new DataView(evt.data);
        var reserved = dv.getUint32(0);
        var classId = dv.getUint16(4);
        var funcId = dv.getUint16(6);
        if (mode === 'static') {
            len = dv.byteLength - 8;
            buffer = new ArrayBuffer(len);
            dvBuff = new DataView(buffer);
            for (i = 0, end = len - 1, asc = 0 <= end; asc ? i <= end : i >= end; asc ? i++ : i--) {
                var asc, end;
                dvBuff.setUint8(i, dv.getUint8(8 + i));
            }
        }
        else {
            len = dv.getUint32(8);
            console.assert(dv.byteLength === (len + 12));
            buffer = new ArrayBuffer(len);
            dvBuff = new DataView(buffer);
            for (i = 0, end1 = len - 1, asc1 = 0 <= end1; asc1 ? i <= end1 : i >= end1; asc1 ? i++ : i--) {
                var asc1, end1;
                dvBuff.setUint8(i, dv.getUint8(12 + i));
            }
        }
        return [dvBuff, classId, funcId];
    };
    Client.prototype._readBase = function (mode, cmd, fn) {
        var _this = this;
        if ((this.websockpool === null || typeof (this.websockpool) === 'undefined')) {
            return fn(null);
        }
        this.websockpool.requestSocket(function (sockid) {
            if (sockid < 0) {
                return fn(null);
            }
            var websocket = _this.websockpool.getSocket(sockid);
            websocket.send(cmd.data);
            websocket.onmessage = function (evt) {
                fn(_this.getPayload(mode, evt)[0]);
                if (_this.websockpool !== null && typeof _this.websockpool !== 'undefined') {
                    _this.websockpool.freeSocket(sockid);
                }
            };
        });
    };
    Client.prototype.readUint32Array = function (cmd, fn) {
        this._readBase('static', cmd, function (data) {
            fn(new Uint32Array(data.buffer));
        });
    };
    Client.prototype.readFloat32Array = function (cmd, fn) {
        this._readBase('static', cmd, function (data) {
            fn(new Float32Array(data.buffer));
        });
    };
    Client.prototype.readFloat64Array = function (cmd, fn) {
        this._readBase('static', cmd, function (data) {
            fn(new Float64Array(data.buffer));
        });
    };
    Client.prototype.readUint32Vector = function (cmd, fn) {
        this._readBase('dynamic', cmd, function (data) {
            fn(new Uint32Array(data.buffer));
        });
    };
    Client.prototype.readFloat32Vector = function (cmd, fn) {
        this._readBase('dynamic', cmd, function (data) {
            fn(new Float32Array(data.buffer));
        });
    };
    Client.prototype.readFloat64Vector = function (cmd, fn) {
        this._readBase('dynamic', cmd, function (data) {
            fn(new Float64Array(data.buffer));
        });
    };
    Client.prototype.readUint32 = function (cmd, fn) {
        this._readBase('static', cmd, function (data) {
            fn(data.getUint32(0));
        });
    };
    Client.prototype.readInt32 = function (cmd, fn) {
        this._readBase('static', cmd, function (data) {
            fn(data.getInt32(0));
        });
    };
    Client.prototype.readFloat32 = function (cmd, fn) {
        this._readBase('static', cmd, function (data) {
            fn(data.getFloat32(0));
        });
    };
    Client.prototype.readFloat64 = function (cmd, fn) {
        this._readBase('static', cmd, function (data) {
            fn(data.getFloat64(0));
        });
    };
    Client.prototype.readBool = function (cmd, fn) {
        this._readBase('static', cmd, function (data) {
            fn(data.getUint8(0) === 1);
        });
    };
    Client.prototype.readTuple = function (cmd, fmt, fn) {
        var _this = this;
        this._readBase('static', cmd, function (data) {
            fn(_this.deserialize(fmt, data));
        });
    };
    Client.prototype.deserialize = function (fmt, dv, onError) {
        if (onError == null) {
            onError = null;
        }
        var tuple = [];
        var offset = 0;
        for (var i = 0, end = fmt.length - 1, asc = 0 <= end; asc ? i <= end : i >= end; asc ? i++ : i--) {
            switch (fmt[i]) {
                case 'B':
                    tuple.push(dv.getUint8(offset));
                    offset += 1;
                    break;
                case 'b':
                    tuple.push(dv.getInt8(offset));
                    offset += 1;
                    break;
                case 'H':
                    tuple.push(dv.getUint16(offset));
                    offset += 2;
                    break;
                case 'h':
                    tuple.push(dv.getInt16(offset));
                    offset += 2;
                    break;
                case 'I':
                    tuple.push(dv.getUint32(offset));
                    offset += 4;
                    break;
                case 'i':
                    tuple.push(dv.getInt32(offset));
                    offset += 4;
                    break;
                case 'f':
                    tuple.push(dv.getFloat32(offset));
                    offset += 4;
                    break;
                case 'd':
                    tuple.push(dv.getFloat64(offset));
                    offset += 8;
                    break;
                case '?':
                    if (dv.getUint8(offset) === 0) {
                        tuple.push(false);
                    }
                    else {
                        tuple.push(true);
                    }
                    offset += 1;
                    break;
                default:
                    this.redirectError(true, "Unknown or unsupported type " + fmt[i], (function () { }), onError);
            }
        }
        return tuple;
    };
    Client.prototype.parseString = function (data, offset) {
        if (offset == null) {
            offset = 0;
        }
        return (__range__(0, data.byteLength - offset - 1, true).map(function (i) { return (String.fromCharCode(data.getUint8(offset + i))); })).join('');
    };
    Client.prototype.readString = function (cmd, fn) {
        var _this = this;
        this._readBase('dynamic', cmd, function (data) {
            fn(_this.parseString(data));
        });
    };
    Client.prototype.readJSON = function (cmd, fn) {
        this.readString(cmd, function (str) {
            fn(JSON.parse(str));
        });
    };
    // ------------------------
    //  Drivers
    // ------------------------
    Client.prototype.loadCmds = function (callback) {
        var _this = this;
        this.readJSON(Command(1, { 'id': 1, 'args': [] }), function (data) {
            for (var _i = 0, data_1 = data; _i < data_1.length; _i++) {
                var driver = data_1[_i];
                var dev = new Driver(driver["class"], driver.id, driver.functions);
                // dev.show()
                _this.driversList.push(dev);
            }
            callback();
        });
    };
    Client.prototype.getDriver = function (name) {
        for (var _i = 0, _a = this.driversList; _i < _a.length; _i++) {
            var driver = _a[_i];
            if (driver.name === name) {
                return driver;
            }
        }
        throw new Error("Driver " + name + " not found");
    };
    Client.prototype.getDriverById = function (id) {
        for (var _i = 0, _a = this.driversList; _i < _a.length; _i++) {
            var driver = _a[_i];
            if (driver.id === id) {
                return driver;
            }
        }
        throw new ReferenceError("Driver ID " + id + " not found");
    };
    return Client;
}());
;
function __range__(left, right, inclusive) {
    var range = [];
    var ascending = left < right;
    var end = !inclusive ? right : ascending ? right + 1 : right - 1;
    for (var i = left; ascending ? i < end : i > end; ascending ? i++ : i--) {
        range.push(i);
    }
    return range;
}
var Imports = (function () {
    function Imports(document) {
        this.importLinks = document.querySelectorAll('link[rel="import"]');
        for (var i = 0; i < this.importLinks.length; i++) {
            var template = this.importLinks[i]["import"].querySelector('.template');
            var clone = document.importNode(template.content, true);
            var parentId = this.importLinks[i].dataset.parent;
            document.getElementById(parentId).appendChild(clone);
        }
    }
    return Imports;
}());
// Interface for the Laser driver
// (c) Koheron
"use strict";
var LaserDriver = (function () {
    function LaserDriver(client) {
        this.client = client;
        this.driver = this.client.getDriver('Laser');
        this.id = this.driver.id;
        this.cmds = this.driver.getCmds();
    }
    LaserDriver.prototype.getStatus = function (cb) {
        this.client.readTuple(Command(this.id, this.cmds['get_status']), '???ffff', function (tup) {
            var status = {};
            status.laser_on = tup[0];
            status.constant_power_on = tup[1];
            status.is_calibrated = tup[2];
            status.current = tup[3];
            status.power = tup[4];
            status.measured_current = tup[5];
            status.measured_power = tup[6];
            cb(status);
        });
    };
    LaserDriver.prototype.start = function () {
        this.client.send(Command(this.id, this.cmds['start']));
    };
    LaserDriver.prototype.stop = function () {
        this.client.send(Command(this.id, this.cmds['stop']));
    };
    LaserDriver.prototype.setCurrent = function (val) {
        this.client.send(Command(this.id, this.cmds['set_current'], val));
    };
    LaserDriver.prototype.setPower = function (val) {
        this.client.send(Command(this.id, this.cmds['set_power'], val));
    };
    LaserDriver.prototype.switchMode = function () {
        this.client.send(Command(this.id, this.cmds['switch_mode']));
    };
    LaserDriver.prototype.calibrate0mW = function () {
        this.client.send(Command(this.id, this.cmds['calibrate_0mW']));
    };
    LaserDriver.prototype.calibrate1mW = function () {
        this.client.send(Command(this.id, this.cmds['calibrate_1mW']));
    };
    return LaserDriver;
}());
var LaserControl = (function () {
    function LaserControl(document, driver) {
        this.document = document;
        this.driver = driver;
        this.laserSwitch = document.getElementById('laser-switch');
        this.initLaserSwitch();
        this.calibrationSpan = document.getElementById('calibration');
        this.calibrationInstructionsDiv = document.getElementById('calibration-instructions');
        this.initStartCalibration();
        this.initEndCalibration();
        this.laserModeSelect = document.getElementById('laser-mode');
        this.initLaserModeSelect();
        this.laserControlInputs = document.getElementsByClassName("laser-control-input");
        this.initLaserControlInputs();
        this.measuredPowerSpan = document.getElementById('measured-power');
        this.canvas = document.getElementById('canvas');
        this.canvas.width = document.querySelector(".laser-control-input[type='range']").offsetWidth;
        this.ctx = this.canvas.getContext("2d");
        this.ctx.fillStyle = 'rgb(100, 100, 100)';
        this.update();
    }
    LaserControl.prototype.update = function () {
        var _this = this;
        this.driver.getStatus(function (status) {
            _this.measuredPowerSpan.innerHTML = Math.max(0, status.measured_power).toFixed(1);
            _this.ctx.clearRect(0, 0, _this.canvas.width, 15);
            _this.ctx.fillRect(0, 0, status.measured_power * _this.canvas.width / 4000, 15);
            if (status.is_calibrated) {
                _this.calibrationSpan.innerHTML = 'Status: Calibrated';
            }
            else {
                _this.calibrationSpan.innerHTML = 'Status: Not calibrated';
            }
            var laserControlInputsArray = [];
            for (var j = 0; j < _this.laserControlInputs.length; j++) {
                laserControlInputsArray.push(_this.laserControlInputs[j]);
            }
            if (laserControlInputsArray.indexOf(document.activeElement) == -1) {
                for (var j = 0; j < _this.laserControlInputs.length; j++) {
                    if (_this.laserControlInputs[j].dataset.command === "setCurrent") {
                        _this.laserControlInputs[j].value = status.current.toFixed(2);
                    }
                    else if (_this.laserControlInputs[j].dataset.command === "setPower") {
                        _this.laserControlInputs[j].value = status.power.toFixed(1);
                    }
                }
            }
            _this.laserOn = status.laser_on;
            _this.laserSwitch.checked = _this.laserOn;
            if (status.constant_power_on) {
                _this.laserModeSelect.value = "power";
                for (var i = 0; i < _this.laserControlInputs.length; i++) {
                    _this.laserControlInputs[i].dataset.command = "setPower";
                    _this.laserControlInputs[i].max = "4000";
                }
            }
            else {
                _this.laserModeSelect.value = 'current';
                for (var i = 0; i < _this.laserControlInputs.length; i++) {
                    _this.laserControlInputs[i].dataset.command = "setCurrent";
                    _this.laserControlInputs[i].max = "40";
                }
            }
            requestAnimationFrame(function () { _this.update(); });
        });
    };
    LaserControl.prototype.initLaserSwitch = function () {
        var _this = this;
        this.laserSwitch.addEventListener('change', function (event) {
            if (_this.laserOn) {
                _this.driver.stop();
                _this.laserOn = false;
            }
            else {
                _this.driver.start();
                _this.laserOn = true;
            }
        });
    };
    LaserControl.prototype.initLaserModeSelect = function () {
        var _this = this;
        this.laserModeSelect.addEventListener('change', function (event) {
            _this.driver.switchMode();
        });
    };
    LaserControl.prototype.initLaserControlInputs = function () {
        var _this = this;
        var events = ['change', 'input'];
        for (var j = 0; j < events.length; j++) {
            for (var i = 0; i < this.laserControlInputs.length; i++) {
                this.laserControlInputs[i].addEventListener(events[j], function (event) {
                    var counterType = "number";
                    if (event.currentTarget.type == "number") {
                        counterType = "range";
                    }
                    var command = event.currentTarget.dataset.command;
                    var value = event.currentTarget.value;
                    document.querySelector(".laser-control-input[data-command='" + command + "'][type='" + counterType + "']").value = value;
                    _this.driver[command](parseFloat(value));
                });
            }
        }
    };
    LaserControl.prototype.initStartCalibration = function () {
        var _this = this;
        var startCalibrationBtn = document.getElementById("start-calibration-btn");
        startCalibrationBtn.addEventListener('click', function (event) {
            _this.driver.calibrate0mW();
            _this.calibrationSpan.style.display = "none";
            _this.calibrationInstructionsDiv.style.display = 'block';
        });
    };
    LaserControl.prototype.initEndCalibration = function () {
        var _this = this;
        var endCalibrationBtn = document.getElementById("end-calibration-btn");
        endCalibrationBtn.addEventListener('click', function (event) {
            _this.driver.calibrate1mW();
            _this.calibrationSpan.style.display = "inline";
            _this.calibrationInstructionsDiv.style.display = 'none';
        });
    };
    return LaserControl;
}());
// DDS Frequency wiget
// (c) Koheron
"use strict";
var DDSFrequency = (function () {
    function DDSFrequency(document, driver) {
        this.driver = driver;
        this.ddsChannelInputs = document.getElementsByClassName("dds-channel-input");
        this.initDDSChannelInputs();
    }
    DDSFrequency.prototype.initDDSChannelInputs = function () {
        var _this = this;
        var events = ['change', 'input'];
        for (var j = 0; j < events.length; j++) {
            for (var i = 0; i < this.ddsChannelInputs.length; i++) {
                this.ddsChannelInputs[i].addEventListener(events[j], function (event) {
                    var counterType = "number";
                    if (event.currentTarget.type == "number") {
                        counterType = "range";
                    }
                    var command = event.currentTarget.dataset.command;
                    var channel = event.currentTarget.dataset.channel;
                    var value = event.currentTarget.value;
                    document.querySelector("[data-command='" + command + "'][data-channel='" + channel + "'][type='" + counterType + "']").value = value;
                    _this.driver[command](channel, 1e6 * parseFloat(value));
                });
            }
        }
    };
    return DDSFrequency;
}());
"use strict";
var App = (function () {
    function App(window, document, ip, plot_placeholder) {
        var _this = this;
        var client = new Client(ip, 5);
        window.addEventListener('HTMLImportsLoaded', function () {
            client.init(function () {
                _this.imports = new Imports(document);
                _this.fft = new FFT(client);
                _this.fft.init(function () {
                    _this.fftApp = new FFTApp(document, _this.fft);
                    _this.ddsFrequency = new DDSFrequency(document, _this.fft);
                    _this.n_pts = _this.fft.fft_size / 2;
                    _this.x_min = 0;
                    _this.x_max = _this.fft.status.fs / 1E6 / 2;
                    _this.y_min = -200;
                    _this.y_max = 170;
                    _this.plotBasics = new PlotBasics(document, plot_placeholder, _this.plot, _this.n_pts, _this.x_min, _this.x_max, _this.y_min, _this.y_max, _this.fft, "", "Frequency (MHz)");
                    _this.plot = new Plot(document, _this.fft, _this.plotBasics);
                    _this.laserDriver = new LaserDriver(client);
                    _this.laserControl = new LaserControl(document, _this.laserDriver);
                    _this.exportFile = new ExportFile(document, _this.plot);
                });
            });
        }, false);
        window.onbeforeunload = function () { client.exit(); };
    }
    return App;
}());
var app = new App(window, document, location.hostname, $('#plot-placeholder'));
// Interface for the FFT driver
// (c) Koheron
"use strict";
var FFT = (function () {
    function FFT(client) {
        this.client = client;
        this.driver = this.client.getDriver('FFT');
        this.id = this.driver.id;
        this.cmds = this.driver.getCmds();
        //this.monitor(1000);
        this.status = {};
        this.status.dds_freq = [];
    }
    FFT.prototype.init = function (cb) {
        var _this = this;
        this.getFFTSize(function (size) {
            _this.fft_size = size;
            _this.getControlParameters(function () {
                cb();
            });
        });
    };
    FFT.prototype.monitor = function (timeout) {
        var _this = this;
        this.getCycleIndex(function (i) {
            setTimeout(function () {
                _this.monitor(timeout);
            }, timeout);
        });
    };
    FFT.prototype.getCycleIndex = function (cb) {
        this.client.readUint32(Command(this.id, this.cmds['get_cycle_index']), function (i) { cb(i); });
    };
    FFT.prototype.getFFTSize = function (cb) {
        this.client.readUint32(Command(this.id, this.cmds['get_fft_size']), function (size) { cb(size); });
    };
    FFT.prototype.read_psd_raw = function (cb) {
        this.client.readFloat32Array(Command(this.id, this.cmds['read_psd_raw']), function (psd) {
            cb(psd);
        });
    };
    FFT.prototype.read_psd = function (cb) {
        this.client.readFloat32Array(Command(this.id, this.cmds['read_psd']), function (psd) {
            cb(psd);
        });
    };
    FFT.prototype.setDDSFreq = function (channel, freq_hz) {
        this.client.send(Command(this.id, this.cmds['set_dds_freq'], channel, freq_hz));
    };
    FFT.prototype.setInputChannel = function (channel) {
        this.client.send(Command(this.id, this.cmds['set_input_channel'], channel));
    };
    FFT.prototype.setFFTWindow = function (windowIndex) {
        this.client.send(Command(this.id, this.cmds['set_fft_window'], windowIndex));
    };
    FFT.prototype.getControlParameters = function (cb) {
        var _this = this;
        this.client.readTuple(Command(this.id, this.cmds['get_control_parameters']), 'dddIdd', function (tup) {
            _this.status.dds_freq[0] = tup[0];
            _this.status.dds_freq[1] = tup[1];
            _this.status.fs = tup[2];
            _this.status.channel = tup[3];
            _this.status.W1 = tup[4];
            _this.status.W2 = tup[5];
            cb(_this.status);
        });
    };
    FFT.prototype.getFFTWindowIndex = function (cb) {
        this.client.readUint32(Command(this.id, this.cmds['get_window_index']), function (windowIndex) {
            cb(windowIndex);
        });
    };
    return FFT;
}());
// Plot widget
// (c) Koheron
"use strict";
var Plot = (function () {
    function Plot(document, fft, plotBasics) {
        this.fft = fft;
        this.plotBasics = plotBasics;
        this.yLabel = "Power Spectral Density";
        this.n_pts = this.fft.fft_size / 2;
        this.x_max = this.fft.status.fs / 1E6 / 2;
        this.peakDatapoint = [];
        this.plot_data = [];
        this.updatePlot(this.x_max);
    }
    Plot.prototype.updatePlot = function (max_x) {
        var _this = this;
        this.fft.read_psd(function (psd) {
            var yUnit = document.querySelector(".unit-input:checked").value;
            _this.peakDatapoint = [max_x / _this.n_pts, _this.convertValue(psd[0], yUnit)];
            for (var i = 0; i <= _this.n_pts; i++) {
                var freq = (i + 1) * max_x / _this.n_pts; // MHz
                var convertedPsd = _this.convertValue(psd[i], yUnit);
                _this.plot_data[i] = [freq, convertedPsd];
            }
            ;
            _this.plotBasics.redraw(_this.plot_data, _this.n_pts, _this.peakDatapoint, _this.yLabel, function () {
                requestAnimationFrame(function () { _this.updatePlot(max_x); });
            });
        });
    };
    Plot.prototype.convertValue = function (inValue, outUnit) {
        // inValue in W / Hz
        var outValue = 0;
        if (outUnit === "dBm-Hz") {
            outValue = 10 * Math.log(inValue / 1E-3) / Math.LN10;
        }
        else if (outUnit === "dBm") {
            outValue = 10 * Math.log(inValue * (this.fft.status.W2 / this.fft.status.W1) * this.fft.status.fs / this.fft.fft_size / 1E-3) / Math.LN10;
        }
        else if (outUnit === "nv-rtHz") {
            outValue = Math.sqrt(50 * inValue) * 1E9;
        }
        return outValue;
    };
    return Plot;
}());
// Plot widget
// (c) Koheron
"use strict";
var PlotBasics = (function () {
    function PlotBasics(document, plot_placeholder, plotDriver, n_pts, x_min, x_max, y_min, y_max, driver, rangeFunction, plotTitle) {
        this.plot_placeholder = plot_placeholder;
        this.plotDriver = plotDriver;
        this.n_pts = n_pts;
        this.x_min = x_min;
        this.x_max = x_max;
        this.y_min = y_min;
        this.y_max = y_max;
        this.driver = driver;
        this.rangeFunction = rangeFunction;
        this.plotTitle = plotTitle;
        this.isPeakDetection = true;
        this.plotTitleSpan = document.getElementById("plot-title");
        this.plotTitleSpan.textContent = this.plotTitle;
        this.range_x = {};
        this.range_x.from = this.x_min;
        this.range_x.to = this.x_max;
        this.range_y = {};
        this.range_y.from = this.y_min;
        this.range_y.to = this.y_max;
        this.setPlot(this.range_x.from, this.range_x.to, this.range_y.from, this.range_y.to);
        this.rangeSelect(this.rangeFunction);
        this.dblClick(this.x_max, this.rangeFunction);
        this.onWheel(this.x_max, this.rangeFunction);
        this.showHoverPoint();
        this.showClickPoint();
        this.plotLeave();
        this.reset_range = true;
        this.hoverDatapointSpan = document.getElementById("hover-datapoint");
        this.hoverDatapoint = [];
        this.clickDatapointSpan = document.getElementById("click-datapoint");
        this.clickDatapoint = [];
        this.peakDatapointSpan = document.getElementById("peak-datapoint");
        this.plot_data = [];
        this.initUnitInputs();
        this.initPeakDetection();
    }
    PlotBasics.prototype.setPlot = function (x_min, x_max, y_min, y_max) {
        this.reset_range = false;
        this.options = {
            canvas: true,
            series: {
                shadowSize: 0 // Drawing is faster without shadows
            },
            yaxis: {
                min: y_min,
                max: y_max
            },
            xaxis: {
                min: x_min,
                max: x_max,
                show: true
            },
            grid: {
                margin: {
                    top: 0,
                    left: 0
                },
                borderColor: "#d5d5d5",
                borderWidth: 1,
                clickable: true,
                hoverable: true,
                autoHighlight: true
            },
            selection: {
                mode: "xy"
            },
            colors: ["#019cd5", "#006400"],
            legend: {
                show: true,
                noColumns: 0,
                labelFormatter: function (label, series) {
                    return "<b style='font-size: 16px; color: #333'>" + label + "\t</b>";
                },
                margin: 0,
                position: "ne"
            }
        };
    };
    PlotBasics.prototype.rangeSelect = function (rangeFunction) {
        var _this = this;
        this.plot_placeholder.bind("plotselected", function (event, ranges) {
            // Clamp the zooming to prevent external zoom
            if (ranges.xaxis.to - ranges.xaxis.from < 0.00001) {
                ranges.xaxis.to = ranges.xaxis.from + 0.00001;
            }
            if (ranges.yaxis.to - ranges.yaxis.from < 0.00001) {
                ranges.yaxis.to = ranges.yaxis.from + 0.00001;
            }
            _this.range_x.from = ranges.xaxis.from;
            _this.range_x.to = ranges.xaxis.to;
            _this.range_y.from = ranges.yaxis.from;
            _this.range_y.to = ranges.yaxis.to;
            if (rangeFunction.length > 0) {
                _this.driver[rangeFunction](ranges.xaxis);
            }
            _this.reset_range = true;
        });
    };
    // A double click on the plot resets to full span
    PlotBasics.prototype.dblClick = function (max_x, rangeFunction) {
        var _this = this;
        this.plot_placeholder.bind("dblclick", function (evt) {
            _this.range_x.from = 0;
            _this.range_x.to = max_x;
            _this.range_y = {};
            if (rangeFunction.length > 0) {
                _this.driver[rangeFunction](_this.range_x);
            }
            _this.reset_range = true;
        });
    };
    PlotBasics.prototype.updateDatapointSpan = function (datapoint, datapointSpan) {
        var positionX = (this.plot.pointOffset({ x: datapoint[0], y: datapoint[1] })).left;
        var positionY = (this.plot.pointOffset({ x: datapoint[0], y: datapoint[1] })).top;
        datapointSpan.innerHTML = "(" + (datapoint[0].toFixed(2)).toString() + "," + datapoint[1].toFixed(2).toString() + ")";
        if (datapoint[0] < (this.range_x.from + this.range_x.to) / 2) {
            datapointSpan.style.left = (positionX + 5).toString() + "px";
        }
        else {
            datapointSpan.style.left = (positionX - 140).toString() + "px";
        }
        if (datapoint[1] < ((this.range_y.from + this.range_y.to) / 2)) {
            datapointSpan.style.top = (positionY - 50).toString() + "px";
        }
        else {
            datapointSpan.style.top = (positionY + 5).toString() + "px";
        }
    };
    PlotBasics.prototype.redraw = function (plot_data, n_pts, peakDatapoint, ylabel, callback) {
        for (var i = 0; i <= n_pts; i++) {
            if (peakDatapoint[1] < plot_data[i][1]) {
                peakDatapoint[0] = plot_data[i][0];
                peakDatapoint[1] = plot_data[i][1];
            }
        }
        var plt_data = [{ label: ylabel, data: plot_data }];
        if (this.reset_range) {
            this.options.xaxis.min = this.range_x.from;
            this.options.xaxis.max = this.range_x.to;
            this.options.yaxis.min = this.range_y.from;
            this.options.yaxis.max = this.range_y.to;
            this.plot = $.plot(this.plot_placeholder, plt_data, this.options);
            this.plot.setupGrid();
            this.range_y.from = this.plot.getAxes().yaxis.min;
            this.range_y.to = this.plot.getAxes().yaxis.max;
            this.reset_range = false;
        }
        else {
            this.plot.setData(plt_data);
            this.plot.draw();
        }
        var localData = this.plot.getData();
        setTimeout(this.plot.unhighlight(), 100);
        if (this.clickDatapoint.length > 0) {
            var i = void 0;
            for (i = 0; i < n_pts; i++) {
                if (localData[0]['data'][i][0] > this.clickDatapoint[0]) {
                    break;
                }
            }
            var p1 = localData[0]['data'][i - 1];
            var p2 = localData[0]['data'][i];
            if (p1 === null) {
                this.clickDatapoint[1] = p2[1];
            }
            else if (p2 === null) {
                this.clickDatapoint[1] = p1[1];
            }
            else {
                this.clickDatapoint[1] = p1[1] + (p2[1] - p1[1]) * (this.clickDatapoint[0] - p1[0]) / (p2[0] - p1[0]);
            }
            if (this.range_x.from < this.clickDatapoint[0] && this.clickDatapoint[0] < this.range_x.to &&
                this.range_y.from < this.clickDatapoint[1] && this.clickDatapoint[1] < this.range_y.to) {
                this.updateDatapointSpan(this.clickDatapoint, this.clickDatapointSpan);
                this.clickDatapointSpan.style.display = "inline-block";
                this.plot.highlight(localData[0], this.clickDatapoint);
            }
            else {
                this.clickDatapointSpan.style.display = "none";
            }
        }
        if (this.isPeakDetection) {
            this.plot.unhighlight(localData[0], peakDatapoint);
            if (this.range_x.from < peakDatapoint[0] && peakDatapoint[0] < this.range_x.to &&
                this.range_y.from < peakDatapoint[1] && peakDatapoint[1] < this.range_y.to) {
                this.updateDatapointSpan(peakDatapoint, this.peakDatapointSpan);
                this.plot.highlight(localData[0], peakDatapoint);
                this.peakDatapointSpan.style.display = "inline-block";
            }
            else {
                this.plot.unhighlight(localData[0], peakDatapoint);
                this.peakDatapointSpan.style.display = "none";
            }
        }
        else {
            this.plot.unhighlight(localData[0], peakDatapoint);
            this.peakDatapointSpan.style.display = "none";
        }
        callback();
    };
    PlotBasics.prototype.redrawRange = function (data, range_x, ylabel, callback) {
        var plt_data = [{ label: ylabel, data: data }];
        if (data.length == 0) {
            callback();
            return;
        }
        if (this.reset_range) {
            this.options.xaxis.min = range_x.from;
            this.options.xaxis.max = range_x.to;
            this.options.yaxis.min = this.range_y.from;
            this.options.yaxis.max = this.range_y.to;
            this.plot = $.plot(this.plot_placeholder, plt_data, this.options);
            this.plot.setupGrid();
            this.reset_range = false;
        }
        else {
            this.plot.setData(plt_data);
            this.plot.draw();
        }
        callback();
    };
    PlotBasics.prototype.redrawTwoChannels = function (ch0, ch1, range_x, label1, label2, is_channel_1, is_channel_2, callback) {
        if (ch0.length === 0 || ch1.length === 0) {
            callback();
            return;
        }
        var plotCh0 = [];
        var plotCh1 = [];
        if (is_channel_1 && is_channel_2) {
            plotCh0 = ch0;
            plotCh1 = ch1;
            // plotData = [ch0, ch1];
        }
        else if (is_channel_1 && !is_channel_2) {
            plotCh0 = ch0;
            plotCh1 = [];
        }
        else if (!is_channel_1 && is_channel_2) {
            plotCh0 = [];
            plotCh1 = ch1;
        }
        else {
            plotCh0 = [];
            plotCh1 = [];
        }
        var plt_data = [{ label: label1, data: plotCh0 }, { label: label2, data: plotCh1 }];
        if (this.reset_range) {
            this.options.xaxis.min = range_x.from;
            this.options.xaxis.max = range_x.to;
            this.options.yaxis.min = this.range_y.from;
            this.options.yaxis.max = this.range_y.to;
            this.plot = $.plot(this.plot_placeholder, plt_data, this.options);
            this.plot.setupGrid();
            this.reset_range = false;
        }
        else {
            this.plot.setData(plt_data);
            this.plot.draw();
        }
        callback();
    };
    PlotBasics.prototype.onWheel = function (max_x, rangeFunction) {
        var _this = this;
        this.plot_placeholder.bind("wheel", function (evt) {
            var delta = evt.originalEvent.deltaX
                + evt.originalEvent.deltaY;
            delta /= Math.abs(delta);
            var zoomRatio = 0.2;
            if (evt.originalEvent.shiftKey) {
                var positionY = evt.originalEvent.pageY - _this.plot.offset().top;
                var y0 = _this.plot.getAxes().yaxis.c2p(positionY);
                _this.range_y = {
                    from: y0 - (1 + zoomRatio * delta) * (y0 - _this.plot.getAxes().yaxis.min),
                    to: y0 - (1 + zoomRatio * delta) * (y0 - _this.plot.getAxes().yaxis.max)
                };
                _this.reset_range = true;
                return false;
            }
            else if (evt.originalEvent.altKey) {
                var positionX = evt.originalEvent.pageX - _this.plot.offset().left;
                var x0 = _this.plot.getAxes().xaxis.c2p(positionX);
                if (x0 < 0 || x0 > max_x) {
                    return;
                }
                _this.range_x = {
                    from: Math.max(x0 - (1 + zoomRatio * delta) * (x0 - _this.plot.getAxes().xaxis.min), 0),
                    to: Math.min(x0 - (1 + zoomRatio * delta) * (x0 - _this.plot.getAxes().xaxis.max), max_x)
                };
                if (rangeFunction.length > 0) {
                    _this.driver[rangeFunction](_this.range_x);
                }
                _this.reset_range = true;
                return false;
            }
            return true;
        });
    };
    PlotBasics.prototype.initUnitInputs = function () {
        var _this = this;
        var unitInputs = document.getElementsByClassName("unit-input");
        for (var i = 0; i < unitInputs.length; i++) {
            unitInputs[i].addEventListener('change', function (event) {
                _this.reset_range = true;
            });
        }
    };
    PlotBasics.prototype.initPeakDetection = function () {
        var _this = this;
        var peakInputs = document.getElementsByClassName("peak-input");
        for (var i = 0; i < peakInputs.length; i++) {
            peakInputs[i].addEventListener('change', function (event) {
                if (_this.isPeakDetection) {
                    _this.isPeakDetection = false;
                }
                else {
                    _this.isPeakDetection = true;
                }
            });
        }
    };
    PlotBasics.prototype.showHoverPoint = function () {
        var _this = this;
        this.plot_placeholder.bind("plothover", function (event, pos, item) {
            if (item) {
                _this.hoverDatapoint[0] = item.datapoint[0];
                _this.hoverDatapoint[1] = item.datapoint[1];
                _this.hoverDatapointSpan.style.display = "inline-block";
                _this.updateDatapointSpan(_this.hoverDatapoint, _this.hoverDatapointSpan);
            }
            else {
                _this.hoverDatapointSpan.style.display = "none";
            }
        });
    };
    PlotBasics.prototype.showClickPoint = function () {
        var _this = this;
        this.plot_placeholder.bind("plotclick", function (event, pos, item) {
            if (item) {
                _this.clickDatapoint[0] = item.datapoint[0];
                _this.clickDatapoint[1] = item.datapoint[1];
                _this.clickDatapointSpan.style.display = "inline-block";
                _this.updateDatapointSpan(_this.clickDatapoint, _this.clickDatapointSpan);
                _this.plot.unhighlight();
                _this.plot.highlight(item.series, _this.clickDatapoint);
            }
        });
    };
    PlotBasics.prototype.plotLeave = function () {
        var _this = this;
        this.plot_placeholder.bind("mouseleave", function (event, pos, item) {
            _this.hoverDatapointSpan.style.display = "none";
        });
    };
    return PlotBasics;
}());
// Control widget
// (c) Koheron
"use strict";
var FFTApp = (function () {
    function FFTApp(document, fft) {
        this.fft = fft;
        this.channelNum = 2;
        this.fftSelects = document.getElementsByClassName("fft-select");
        this.initFFTSelects();
        this.fftInputs = document.getElementsByClassName("fft-input");
        this.initFFTInputs();
        this.updateFFTWindowInputs();
        this.updateControls();
    }
    // Updaters
    FFTApp.prototype.updateControls = function () {
        var _this = this;
        this.fft.getControlParameters(function (sts) {
            for (var i = 0; i < _this.channelNum; i++) {
                var inputs = document.querySelectorAll(".dds-channel-input[data-command='setDDSFreq'][data-channel='" + i.toString() + "']");
                var inputsArray = [];
                for (var j = 0; j < inputs.length; j++) {
                    inputsArray.push(inputs[j]);
                }
                if (inputsArray.indexOf(document.activeElement) == -1) {
                    for (var j = 0; j < inputs.length; j++) {
                        inputs[j].value = (sts.dds_freq[i] / 1e6).toFixed(6);
                        if (inputs[j].type == "range") {
                            inputs[j].max = (sts.fs / 1e6 / 2).toFixed(1);
                        }
                    }
                }
            }
            document.querySelector("[data-command='setInputChannel'][value='" + sts.channel.toString() + "']").checked = true;
            requestAnimationFrame(function () { _this.updateControls(); });
        });
    };
    FFTApp.prototype.updateFFTWindowInputs = function () {
        var _this = this;
        this.fft.getFFTWindowIndex(function (windowIndex) {
            document.querySelector("[data-command='setFFTWindow']").value = windowIndex.toString();
            requestAnimationFrame(function () { _this.updateFFTWindowInputs(); });
        });
    };
    // Setters
    FFTApp.prototype.initFFTSelects = function () {
        var _this = this;
        for (var i = 0; i < this.fftSelects.length; i++) {
            this.fftSelects[i].addEventListener('change', function (event) {
                _this.fft[event.currentTarget.dataset.command](event.currentTarget.value);
            });
        }
    };
    FFTApp.prototype.initFFTInputs = function () {
        var _this = this;
        for (var i = 0; i < this.fftInputs.length; i++) {
            this.fftInputs[i].addEventListener('change', function (event) {
                _this.fft[event.currentTarget.dataset.command](event.currentTarget.value);
            });
        }
    };
    return FFTApp;
}());
// Export file widget
// (c) Koheron
"use strict";
var ExportFile = (function () {
    function ExportFile(document, plot_) {
        this.plot_ = plot_;
        this.exportDataButtons = document.getElementsByClassName("export-data");
        this.exportPlotButtons = document.getElementsByClassName("export-plot");
        this.initExportData();
        this.initExportPlot();
    }
    ExportFile.prototype.initExportData = function () {
        var _this = this;
        var _loop_1 = function (i) {
            this_1.exportDataButtons[i].addEventListener('click', function (event) {
                var csvContent = "data:text/csv;charset=utf-8,";
                var dateTime = new Date();
                var fftWindowIndex = document.querySelector("[data-command='setFFTWindow']").value;
                var inputChannel = document.querySelector("[name='input-channel']:checked").value;
                var ddsInputs = document.querySelectorAll(".dds-channel-input[type='range']");
                csvContent += "Koheron FFT \n";
                csvContent += dateTime.getDate() + "/" + (dateTime.getMonth() + 1) + "/" + dateTime.getFullYear() + " ";
                csvContent += dateTime.getHours() + ":" + dateTime.getMinutes() + ":" + dateTime.getSeconds() + "\n\n";
                csvContent += '"Window",' + fftWindowIndex + "\n";
                csvContent += '"Input channel",' + inputChannel + "\n";
                for (var i_1 = 0; i_1 < ddsInputs.length; i_1++) {
                    var channel = ddsInputs[i_1].dataset.channel;
                    csvContent += '"Channel ' + channel + ' DDS frequency (MHz)",' + ddsInputs[i_1].value + "\n";
                }
                var yUnit = document.querySelector(".unit-input:checked").value;
                csvContent += '"Frequency (MHz)",' + _this.plot_.yLabel + ' (' + yUnit.replace("-", "/") + ')" \n';
                _this.plot_.plot_data.forEach(function (rowArray) {
                    var row = rowArray.join(",");
                    csvContent += row + "\n";
                });
                var exportGroup = _this.exportDataButtons[i].parentElement;
                var exportLink = (exportGroup.getElementsByTagName("a")[0]);
                exportLink.href = encodeURI(csvContent);
                exportLink.click();
            });
        };
        var this_1 = this;
        for (var i = 0; i < this.exportDataButtons.length; i++) {
            _loop_1(i);
        }
    };
    ExportFile.prototype.initExportPlot = function () {
        var _this = this;
        var _loop_2 = function (i) {
            this_2.exportPlotButtons[i].addEventListener('click', function (event) {
                console.log("click");
                console.log(_this.plot_);
                var canvas = _this.plot_.plotBasics.plot.getCanvas();
                console.log("canvas");
                var imagePng = canvas.toDataURL("image/png").replace("image/png", "image/octet-stream");
                var exportGroup = _this.exportPlotButtons[i].parentElement;
                var exportLink = (exportGroup.getElementsByTagName("a")[0]);
                exportLink.href = imagePng;
                exportLink.click();
            });
        };
        var this_2 = this;
        for (var i = 0; i < this.exportPlotButtons.length; i++) {
            _loop_2(i);
        }
    };
    return ExportFile;
}());
